﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using DevLeap.SP2010.RESTConsumer.DevBookServiceReference;

namespace DevLeap.SP2010.RESTConsumer
{
    class Program
    {
        static void Main(string[] args)
        {
            // FirstSimpleCollectionEnumeration();
            // LINQQueryOverItems();
            // LINQQueryWithJoin();
            // LINQQueryWithPaging();
            // UpdateItem();
            // AddItem();
            // DeleteItem();
        }

        // Listing 5-29
        static void FirstSimpleCollectionEnumeration()
        {
            SharePoint2010DeveloperReferenceBookDataContext dc =
                new SharePoint2010DeveloperReferenceBookDataContext(new Uri("http://devbook.sp2010.local/_vti_bin/ListData.svc"));
            dc.Credentials = System.Net.CredentialCache.DefaultCredentials;

            foreach (var item in dc.DevLeapContacts)
            {
                Console.WriteLine(item);
            }
        }

        // Listing 5-30
        static void LINQQueryOverItems()
        {
            SharePoint2010DeveloperReferenceBookDataContext dc =
                new SharePoint2010DeveloperReferenceBookDataContext(new Uri("http://devbook.sp2010.local/_vti_bin/ListData.svc"));
            dc.Credentials = System.Net.CredentialCache.DefaultCredentials;

            var query = from c in dc.DevLeapContacts
                        where c.ContentType == "DevLeapCustomer"
                        select new { c.ContactID, c.Title, c.CompanyName, c.CustomerLevelValue };

            foreach (var item in query)
            {
                Console.WriteLine(item);
            }
        }

        // Listing 5-31
        static void LINQQueryWithJoin()
        {
            SharePoint2010DeveloperReferenceBookDataContext dc =
                new SharePoint2010DeveloperReferenceBookDataContext(new Uri("http://devbook.sp2010.local/_vti_bin/ListData.svc"));
            dc.Credentials = System.Net.CredentialCache.DefaultCredentials;

            // This query does not work, because join is not supported
            var query = from c in dc.DevLeapContacts
                        where c.ContentType == "DevLeapCustomer"
                        join i in dc.Invoices on c.Id equals i.InvoiceCustomerLookupId
                        select new { c.ContactID, c.Title, c.CompanyName, i.Name };

            foreach (var item in query)
            {
                Console.WriteLine(item);
            }
        }

        // Listing 5-32
        static void LINQQueryWithPaging()
        {
            SharePoint2010DeveloperReferenceBookDataContext dc =
                new SharePoint2010DeveloperReferenceBookDataContext(new Uri("http://devbook.sp2010.local/_vti_bin/ListData.svc"));
            dc.Credentials = System.Net.CredentialCache.DefaultCredentials;

            // Get the second page, with a page size of 10
            var query = (from c in dc.ContactsCOM
                        select c).Skip(10).Take(10);

            foreach (var item in query)
            {
                Console.WriteLine(item.EMailAddress);
            }
        }

        // Listing 5-33
        static void UpdateItem()
        {
            SharePoint2010DeveloperReferenceBookDataContext dc =
                new SharePoint2010DeveloperReferenceBookDataContext(new Uri("http://devbook.sp2010.local/_vti_bin/ListData.svc"));
            dc.Credentials = System.Net.CredentialCache.DefaultCredentials;

            DevLeapContactsItem item = (from c in dc.DevLeapContacts
                                        where c.Title == "Paolo Pialorsi"
                                        select c).First();

            item.CompanyName += " - Changed!";
            dc.UpdateObject(item);

            dc.SaveChanges();
        }

        // Listing 5-34
        static void AddItem()
        {
            SharePoint2010DeveloperReferenceBookDataContext dc =
                new SharePoint2010DeveloperReferenceBookDataContext(new Uri("http://devbook.sp2010.local/_vti_bin/ListData.svc"));
            dc.Credentials = System.Net.CredentialCache.DefaultCredentials;

            DevLeapContactsItem item = new DevLeapContactsItem { 
                Title = "Sample Customer",
                ContactID = "CC001",
                ContentType = "DevLeapCustomer",
                CompanyName = "Sample Company",
                CountryValue = "Germany",
                CustomerLevelValue = "Level A"
            };

            dc.AddToDevLeapContacts(item);

            dc.SaveChanges();
        }

        // Listing 5-35
        static void DeleteItem()
        {
            SharePoint2010DeveloperReferenceBookDataContext dc =
                new SharePoint2010DeveloperReferenceBookDataContext(new Uri("http://devbook.sp2010.local/_vti_bin/ListData.svc"));
            dc.Credentials = System.Net.CredentialCache.DefaultCredentials;

            DevLeapContactsItem item = (from c in dc.DevLeapContacts
                                        where c.ContactID == "CC001"
                                        select c).First();

            dc.DeleteObject(item);

            dc.SaveChanges();
        }
    }
}
