﻿using System;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using System.Linq;
using System.Xml.Linq;
using System.Xml;
using System.Collections.Generic;

namespace DevLeap.SP2010.AdvancedWebParts.AjaxTreeOfProductsWithCategories
{
    [ToolboxItemAttribute(false)]
    public class AjaxTreeOfProductsWithCategories : WebPart
    {
        protected UpdatePanel ajaxPanel;
        protected TreeView productsTree;

        [WebBrowsable(true)]
        [Personalizable(true)]
        public String XmlDataSourceUri { get; set; }

        protected override void CreateChildControls()
        {
            if (!String.IsNullOrEmpty(this.XmlDataSourceUri))
            {
                SPWeb web = SPControl.GetContextWeb(HttpContext.Current);
                SPFile file = web.GetFile(this.XmlDataSourceUri);

                XElement ds = XElement.Load(XmlReader.Create(file.OpenBinaryStream()));

                this.productsTree = new TreeView();
                this.productsTree.DataSource =
                    from XElement x in ds.Descendants("category")
                    select new CategoryItem
                    {
                        Id = x.Attribute("id").Value.ToString(),
                        Description = x.Attribute("description").Value.ToString(),
                        Products =
                            from p in x.Descendants("product")
                            select new ProductItem
                            {
                                Code = x.Attribute("code").Value.ToString(),
                                Description = x.Attribute("description").Value.ToString(),
                                CategoryId = x.Attribute("categoryId").Value.ToString(),
                                Price = Decimal.Parse(x.Attribute("price").Value),
                            },
                    };
                this.productsTree.DataBindings.Add(new TreeNodeBinding
                {
                    DataMember = "CategoryItem",
                    PopulateOnDemand = false,
                    TextField = "Description",
                    ValueField = "Id"
                });
                this.productsTree.DataBindings.Add(new TreeNodeBinding
                {
                    DataMember = "ProductItem",
                    PopulateOnDemand = true,
                    TextField = "Description",
                    ValueField = "Code"
                });
                this.productsTree.DataBind();

                this.ajaxPanel = new UpdatePanel();
                this.ajaxPanel.UpdateMode = UpdatePanelUpdateMode.Always;
                this.ajaxPanel.ContentTemplateContainer.Controls.Add(this.productsTree);
                this.Controls.Add(this.ajaxPanel);
            }
            else
            {
                this.Controls.Add(new LiteralControl("Please configure the Web Part data source URI"));
            }
        }

        private class CategoryItem
        {
            public String Id { get; set; }
            public String Description { get; set; }
            public IEnumerable<ProductItem> Products { get; set; }
        }

        private class ProductItem
        {
            public String Code { get; set; }
            public String Description { get; set; }
            public String CategoryId { get; set; }
            public Decimal Price { get; set; }
        }
    }
}
