﻿using System;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using System.Xml;

namespace DevLeap.SP2010.AdvancedWebParts.AjaxTreeProductsWebPart
{
    [ToolboxItemAttribute(false)]
    public class AjaxTreeProductsWebPart : WebPart
    {
        protected UpdatePanel ajaxPanel;
        protected TreeView treeProducts;
        private IEnumerable<CategoryItem> categoriesWithProducts;

        [WebBrowsable(true)]
        [Personalizable(true)]
        public String XmlDataSourceUri { get; set; }

        protected override void CreateChildControls()
        {
            if (!String.IsNullOrEmpty(this.XmlDataSourceUri))
            {
                SPWeb web = SPControl.GetContextWeb(HttpContext.Current);
                SPFile file = web.GetFile(this.XmlDataSourceUri);

                XElement ds = XElement.Load(XmlReader.Create(file.OpenBinaryStream()));

                categoriesWithProducts =
                    from XElement c in ds.Descendants("category")
                    select new CategoryItem
                    {
                        Id = c.Attribute("id").Value.ToString(),
                        Description = c.Attribute("description").Value.ToString(),
                        Products =
                            from XElement p in ds.Descendants("product")
                            where p.Attribute("categoryId").Value.ToString() == c.Attribute("id").Value.ToString()
                            select new ProductItem
                            {
                                Code = p.Attribute("code").Value.ToString(),
                                Description = p.Attribute("description").Value.ToString(),
                                CategoryId = p.Attribute("categoryId").Value.ToString(),
                                Price = Decimal.Parse(p.Attribute("price").Value),
                            }
                    };

                this.treeProducts = new TreeView();

                foreach (var c in categoriesWithProducts)
                {
                    treeProducts.Nodes.Add(new TreeNode(c.Description, c.Id) { PopulateOnDemand = true });
                }

                this.treeProducts.ExpandDepth = 0;
                this.treeProducts.TreeNodePopulate += new TreeNodeEventHandler(treeProducts_TreeNodePopulate);
                this.ajaxPanel = new UpdatePanel();
                this.ajaxPanel.ContentTemplateContainer.Controls.Add(this.treeProducts);

                this.Controls.Add(this.ajaxPanel);
            }
            else
            {
                this.Controls.Add(new LiteralControl("Please configure the Web Part data source URI"));
            }
        }

        void treeProducts_TreeNodePopulate(object sender, TreeNodeEventArgs e)
        {
            EnsureChildControls();

            foreach (var p in categoriesWithProducts.First(c => c.Id == e.Node.Value).Products)
            {
                e.Node.ChildNodes.Add(new TreeNode(p.Description, p.Code));
            }
        }

        private class CategoryItem
        {
            public String Id { get; set; }
            public String Description { get; set; }
            public IEnumerable<ProductItem> Products { get; set; }
        }

        private class ProductItem
        {
            public String Code { get; set; }
            public String Description { get; set; }
            public String CategoryId { get; set; }
            public Decimal Price { get; set; }
        }
    }
}
