﻿using System;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using DevLeap.SP2010.AdvancedWebParts.BingServiceAgent.LiveSearchServiceReference;
using System.ServiceModel;

namespace DevLeap.SP2010.AdvancedWebParts.BingAsynchronousWebPart
{
    [ToolboxItemAttribute(false)]
    public class BingAsynchronousWebPart : WebPart
    {
        [WebBrowsable(true), Personalizable(true)]
        public String AppID { get; set; }

        [WebBrowsable(true), Personalizable(true)]
        public String BingSearchServiceUri { get; set; }

        protected TextBox searchText;
        protected Button searchCommandAsynchronously;
        protected Button searchCommandSynchronously;
        protected GridView searchResults;

        private LiveSearchPortTypeClient bing;

        private DateTime startTime;
        private DateTime endTime;

        protected override void OnLoad(EventArgs e)
        {
            this.Page.PreRenderComplete += new EventHandler(Page_PreRenderComplete);

            startTime = DateTime.Now;

            base.OnLoad(e);
        }

        private void ensureBingSearchClient()
        {
            if (this.bing == null || this.bing.State != CommunicationState.Opened)
            {
                this.bing = new LiveSearchPortTypeClient(new BasicHttpBinding(), new EndpointAddress(this.BingSearchServiceUri));
                this.bing.Open();
            }
        }

        protected override void CreateChildControls()
        {
            this.searchText = new TextBox();
            this.searchText.Columns = 40;
            this.Controls.Add(new LiteralControl("Text to search:"));
            this.Controls.Add(this.searchText);

            this.searchCommandSynchronously = new Button();
            this.searchCommandSynchronously.Text = "Search Sychronously";
            this.searchCommandSynchronously.Click += new EventHandler(searchCommandSynchronously_Click);
            this.Controls.Add(new LiteralControl("<br/>"));
            this.Controls.Add(this.searchCommandSynchronously);

            this.searchCommandAsynchronously = new Button();
            this.searchCommandAsynchronously.Text = "Search Asychronously";
            this.searchCommandAsynchronously.Click += new EventHandler(searchCommandAsynchronously_Click);
            this.Controls.Add(new LiteralControl("&nbsp;&nbsp;"));
            this.Controls.Add(this.searchCommandAsynchronously);

            this.searchResults = new GridView();
            this.Controls.Add(this.searchResults);
        }

        void searchCommandSynchronously_Click(object sender, EventArgs e)
        {
            ensureBingSearchClient();

            SearchResponse response = bing.Search(new SearchRequest { 
                AppId = this.AppID,
                Query = this.searchText.Text, 
                UILanguage = "en-US",
                Options = new SearchOption[] { SearchOption.DisableLocationDetection },
                Sources = new SourceType[] { SourceType.News } });

            if (response.News != null && response.News.Results != null)
            {
                this.searchResults.DataSource = response.News.Results;
                this.searchResults.DataBind();
            }
        }

        IAsyncResult BeginSearch(Object sender, EventArgs e, AsyncCallback cb, Object state)
        {
            ensureBingSearchClient();

            SearchRequest request = new SearchRequest
            {
                AppId = this.AppID,
                Query = this.searchText.Text,
                UILanguage = "en-US",
                Options = new SearchOption[] { SearchOption.DisableLocationDetection },
                Sources = new SourceType[] { SourceType.News }
            };

            return (bing.BeginSearch(request, cb, state));
        }

        void EndSearch(IAsyncResult asyncResult)
        {
            SearchResponse response = bing.EndSearch(asyncResult);

            if (response.News != null && response.News.Results != null)
            {
                this.searchResults.DataSource = response.News.Results;
                this.searchResults.DataBind();
            }
        }

        void TimeoutSearch(IAsyncResult asyncResult)
        {
            this.Controls.Add(new LiteralControl("<br/>Timeout expired!"));
        }

        void searchCommandAsynchronously_Click(object sender, EventArgs e)
        {
            this.Page.RegisterAsyncTask(new PageAsyncTask(BeginSearch, EndSearch, TimeoutSearch, null));
            this.Controls.Add(new LiteralControl(String.Format("<br/>{0} - In the mean time I can do something else ...", DateTime.Now)));
        }


        void Page_PreRenderComplete(object sender, EventArgs e)
        {
            endTime = DateTime.Now;
            this.Controls.Add(new LiteralControl(String.Format("<br/>Start: {0} - End: {1}", startTime, endTime)));
            base.OnPreRender(e);
        }
    }
}
