﻿using System;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using System.Xml.Xsl;
using System.Xml;
using System.IO;

namespace DevLeap.SP2010.AdvancedWebParts.RSSFeedDynamicViewerWebPart
{
    [ToolboxItemAttribute(false)]
    public class RSSFeedDynamicViewerWebPart : WebPart
    {
        [WebBrowsable(true)]
        [Personalizable(true)]
        public String RSSFeedUri { get; set; }

        [WebBrowsable(true)]
        [Personalizable(true)]
        public String XsltUri { get; set; }

        protected IButtonControl selectItemCommand;
        protected ListControl postsList;

        protected override void CreateChildControls()
        {
            if (this.WebPartManager.DisplayMode == WebPartManager.BrowseDisplayMode)
            {
                if (!String.IsNullOrEmpty(this.RSSFeedUri) && !String.IsNullOrEmpty(this.XsltUri))
                {
                    XslCompiledTransform xslt = new XslCompiledTransform();
                    
                    SPFile xsltFile = SPContext.Current.Web.GetFile(this.XsltUri);
                    if (xsltFile != null)
                    {
                        xslt.Load(new XmlTextReader(xsltFile.OpenBinaryStream()));

                        XmlReader xml = XmlReader.Create(this.RSSFeedUri, new XmlReaderSettings
                        {
                            CloseInput = true,
                            ValidationType = System.Xml.ValidationType.None,
                            XmlResolver = new XmlUrlResolver()
                        });

                        using (xml)
                        {
                            StringWriter writer = new StringWriter();
                            xslt.Transform(xml, null, writer);

                            Control parsedControl = this.Page.ParseControl(writer.ToString());
                            this.Controls.Add(parsedControl);

                            this.selectItemCommand = this.FindControl("selectItemCommand") as IButtonControl;
                            this.postsList = this.FindControl("postsList") as ListControl;
                            if (this.selectItemCommand != null)
                            {
                                this.selectItemCommand.Click += new EventHandler(selectItemCommand_Click);
                            }
                        }
                    }
                    else
                    {
                        this.Controls.Add(new LiteralControl("Please provide a valid XSLT file URI in the current site."));
                    }
                }
                else
                {
                    this.Controls.Add(new LiteralControl("Please configure the Web Part."));
                }
            }
            else
            {
                this.Controls.Add(new LiteralControl("This Web Part renders its output only in browsing mode."));
            }
        }

        void selectItemCommand_Click(object sender, EventArgs e)
        {
            if (this.postsList != null)
            {
                if (this.postsList.SelectedValue != null)
                {
                    this.Page.Response.Redirect(this.postsList.SelectedValue);
                }
            }
        }
    }
}
