using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;

namespace DevLeap.SP2010.SampleFeature.Features.SampleWebPart
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("80391627-235b-4a4c-862f-da07b8765f0c")]
    public class SampleWebPartEventReceiver : SPFeatureReceiver
    {
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            // Get the parent of the feature
            // Current feature has a Site scope, thus the Parent should be a Site Collection
            SPSite site = properties.Feature.Parent as SPSite;

            if (site != null)
            {
                SPWeb web = site.RootWeb;

                // Check to see if the list already exists
                try
                {
                    SPList targetList = web.Lists["Sample List"];
                }
                catch (ArgumentException)
                {
                    // The list does not exist, thus you can create it
                    Guid listId = web.Lists.Add("Sample List", "Sample List for SampleWebPart", SPListTemplateType.Events);
                    SPList list = web.Lists[listId];
                    list.OnQuickLaunch = true;
                    list.Update();
                }
            }
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            // Get the parent of the feature
            // Current feature has a Site scope, thus the Parent should be a Site Collection
            SPSite site = properties.Feature.Parent as SPSite;

            if (site != null)
            {
                SPWeb web = site.RootWeb;

                // Check to see if the list already exists
                try
                {
                    SPList list = web.Lists["Sample List"];
                    list.Delete();
                }
                catch (ArgumentException)
                {
                    // The list does not exist, thus you don't need to delete it
                }
            }
        }

        public override void FeatureUpgrading(SPFeatureReceiverProperties properties, 
            string upgradeActionName, 
            System.Collections.Generic.IDictionary<string, string> parameters)
        {
        }
    }
}
