﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using System.Threading;
using System.Xml.Linq;

namespace DevLeap.SP2010.CustomFields
{
    public class IBANField : SPFieldMultiColumn
    {
        public IBANField(SPFieldCollection fields, String fieldName)
            : base(fields, fieldName)
        {
        }

        public IBANField(SPFieldCollection fields, String typeName, String displayName)
            : base(fields, typeName, displayName)
        {
        }

        #region Custom configuration code section

        private Guid _banksListID;

        public Guid BanksListID 
        {
            get 
            {
                if (this._banksListID == Guid.Empty)
                {
                    String currentBanksListIDPropertyValue = this.GetProperty("BanksListID");
                    if (!String.IsNullOrEmpty(currentBanksListIDPropertyValue))
                        this._banksListID = new Guid(currentBanksListIDPropertyValue);
                }
                return (this._banksListID);
            }
            set 
            {
                this._banksListID = value;
                SetPropertyOnThread("BanksListID", this._banksListID);
            } 
        }

        private void SetPropertyOnThread(string propertyName, Object value)
        {
            Thread.SetData(Thread.GetNamedDataSlot(propertyName), value);
        }

        private TProperty GetPropertyFromThread<TProperty>(string propertyName)
        {
            return (TProperty)Thread.GetData(Thread.GetNamedDataSlot(propertyName));
        }

        private void CleanThreadLocalStorage()
        {
            Thread.FreeNamedDataSlot("BanksListID");
        }

        private bool savingSchemaXml;
 
        public override void OnAdded(SPAddFieldOptions op)
        {
            if (!this.savingSchemaXml)
            {
                this.savingSchemaXml = true;
                base.SchemaXml = this.CreateOrUpdateFieldSchemaXml(op);
                this.CleanThreadLocalStorage();
            }
        }

        public override void OnUpdated()
        {
            SPAddFieldOptions op = SPAddFieldOptions.Default;

            String addFieldOptionPropertyValue = this.GetProperty("AddFieldOption");
            if (!String.IsNullOrEmpty(addFieldOptionPropertyValue))
            {
                op = (SPAddFieldOptions)Enum.Parse(typeof(SPAddFieldOptions), this.GetProperty("AddFieldOption"), true);
            }
            if (!this.savingSchemaXml)
            {
                this.savingSchemaXml = true;
                base.SchemaXml = this.CreateOrUpdateFieldSchemaXml(op);
                this.CleanThreadLocalStorage();
            }
        }

        public override void OnDeleting()
        {
            base.OnDeleting();
        }

        private string CreateOrUpdateFieldSchemaXml(SPAddFieldOptions op)
        {
            XElement schemaXmlElement = XElement.Parse(this.SchemaXml);
            schemaXmlElement.SetAttributeValue("BanksListID", this.GetPropertyFromThread<Guid>("BanksListID"));
            schemaXmlElement.SetAttributeValue("AddFieldOption", op.ToString());

            return (schemaXmlElement.ToString());
        }

        #endregion

        public override Type FieldValueType
        {
            get
            {
                return (typeof(IBANFieldValue));
            }
        }

        public override object PreviewValueTyped
        {
            get
            {
                IBANFieldValue previewValue = new IBANFieldValue(";#IT;#12;#A;#12345;#12345;#123456789012;#");
                return(previewValue);
            }
        }

        public override Microsoft.SharePoint.WebControls.BaseFieldControl FieldRenderingControl
        {
            get
            {
                BaseFieldControl renderingControl = new IBANFieldControl();
                renderingControl.FieldName = this.InternalName;

                return renderingControl;
            }
        }

        public override Microsoft.SharePoint.MobileControls.SPMobileBaseFieldControl FieldRenderingMobileControl
        {
            get
            {
                return(new IBANMobileFieldControl());
            }
        }

        public override object GetFieldValue(string value)
        {
            if (!String.IsNullOrEmpty(value))
                return (new IBANFieldValue(value));
            else
                return (null);
        }

        public override string GetFieldValueAsText(object value)
        {
            IBANFieldValue typedValue;
            if (value == null)
            {
                return string.Empty;
            }
            if (value is IBANFieldValue)
            {
                typedValue = (IBANFieldValue)value;
            }
            else
            {
                if (!(value is string))
                {
                    throw new ArgumentException();
                }
                typedValue = new IBANFieldValue(((string)value).Replace(" ", ";#"));
            }

            return (String.Format("{0} {1} {2} {3} {4} {5}",
                typedValue.CountryCode, typedValue.CheckDigit,
                typedValue.CIN, typedValue.ABI, typedValue.CAB,
                typedValue.AccountNumber
                ));
        }
    }
}
