﻿// Uncomment the following line to render display by code
// #define RENDERDISPLAYBYCODE

// Uncomment the following line to render display by ASCX template control
#define RENDERBYTEMPLATES

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint.WebControls;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Diagnostics;
using Microsoft.SharePoint;

namespace DevLeap.SP2010.CustomFields
{
    public class IBANFieldControl: BaseFieldControl
    {
        private InputFormTextBox countryCode;
        private InputFormTextBox checkDigit;
        private InputFormTextBox CIN;
        private InputFormTextBox ABI;
        private InputFormTextBox CAB;
        private InputFormTextBox accountNumber;

#if RENDERBYTEMPLATES

        private Label IBANValue;

        public IBANFieldControl()
            : base()
        {
            this.DisplayTemplateName = "DevLeapIBANFieldDisplay";
        }
#endif

        protected override void CreateChildControls()
        {
            base.CreateChildControls();

#if !RENDERBYTEMPLATES
            if ((this.ControlMode == SPControlMode.Edit) || (this.ControlMode == SPControlMode.New))
            {
                this.countryCode = new InputFormTextBox();
                this.countryCode.Columns = 2;
                this.countryCode.MaxLength = 2;
                this.Controls.Add(new LiteralControl("Country Code: "));
                this.Controls.Add(this.countryCode);

                this.checkDigit = new InputFormTextBox();
                this.checkDigit.Columns = 2;
                this.checkDigit.MaxLength = 2;
                this.Controls.Add(new LiteralControl("<br />Check Digit: "));
                this.Controls.Add(this.checkDigit);

                this.CIN = new InputFormTextBox();
                this.CIN.Columns = 1;
                this.CIN.MaxLength = 1;
                this.Controls.Add(new LiteralControl("<br />CIN: "));
                this.Controls.Add(this.CIN);

                this.ABI = new InputFormTextBox();
                this.ABI.Columns = 5;
                this.ABI.MaxLength = 5;
                this.Controls.Add(new LiteralControl("<br />ABI: "));
                this.Controls.Add(this.ABI);

                this.CAB = new InputFormTextBox();
                this.CAB.Columns = 5;
                this.CAB.MaxLength = 5;
                this.Controls.Add(new LiteralControl("<br />CAB: "));
                this.Controls.Add(this.CAB);

                this.accountNumber = new InputFormTextBox();
                this.accountNumber.Columns = 12;
                this.accountNumber.MaxLength = 12;
                this.Controls.Add(new LiteralControl("<br />Account Number: "));
                this.Controls.Add(this.accountNumber);
            }
#endif
#if RENDERBYTEMPLATES
            if ((this.ControlMode == SPControlMode.Edit) || (this.ControlMode == SPControlMode.New))
            {
                this.countryCode = (InputFormTextBox)TemplateContainer.FindControl("countryCode");
                this.checkDigit = (InputFormTextBox)TemplateContainer.FindControl("checkDigit");
                this.CIN = (InputFormTextBox)TemplateContainer.FindControl("CIN");
                this.ABI = (InputFormTextBox)TemplateContainer.FindControl("ABI");
                this.CAB = (InputFormTextBox)TemplateContainer.FindControl("CAB");
                this.accountNumber = (InputFormTextBox)TemplateContainer.FindControl("accountNumber");
            }
            else
            {
                this.IBANValue = (Label)TemplateContainer.FindControl("IBANValue");
            }
#endif
        }

#if RENDERDISPLAYBYCODE
        protected override void RenderFieldForDisplay(HtmlTextWriter output)
        {
            this.EnsureChildControls();
            output.Write(this.Field.GetFieldValueAsText(this.ListItemFieldValue));
        }
#endif

#if RENDERBYTEMPLATES
        protected override void RenderFieldForDisplay(HtmlTextWriter output)
        {
            this.EnsureChildControls();
            if (this.IBANValue != null)
                this.IBANValue.Text = this.Field.GetFieldValueAsText(this.ListItemFieldValue).Replace(" ", "-");

            base.RenderFieldForDisplay(output);
        }
#endif

        public override void Validate()
        {
            IBANField currentField = this.Field as IBANField;
            if (currentField != null)
            { 
                SPWeb web = SPContext.Current.Web;

                try
                {
                    SPList lookupList = web.Lists[currentField.BanksListID];

                    SPQuery query = new SPQuery();
                    query.Query = String.Format("<Where><And><Eq><FieldRef Name=\"DevLeapBankABI\" /><Value Type=\"Number\">{0}</Value></Eq><Eq><FieldRef Name=\"DevLeapBankCAB\" /><Value Type=\"Number\">{1}</Value></Eq></And></Where></Query>",
                        this.ABI.Text, this.CAB.Text);
                    SPListItemCollection items = lookupList.GetItems(query);

                    if (items.Count == 0)
                    {
                        this.IsValid = false;
                        this.ErrorMessage = "Invalid ABI/CAB values";
                    }
                }
                catch (IndexOutOfRangeException)
                {
                    this.IsValid = false;
                    this.ErrorMessage = "Invalid Banks lookup list ID";
                }
            }
       }

#if RENDERBYTEMPLATES
        protected override string DefaultTemplateName
        {
            get
            {
                return "DevLeapIBANFieldEditAdd";
            }
        }
#endif

        public override object Value
        {
            get
            {
                return new IBANFieldValue {
                    CountryCode = this.countryCode.Text,
                    CheckDigit = this.checkDigit.Text,
                    CIN = this.CIN.Text,
                    ABI = this.ABI.Text,
                    CAB = this.CAB.Text,
                    AccountNumber = this.accountNumber.Text,
                };
            }
            set
            {
                IBANFieldValue typedValue = value as IBANFieldValue;

                if (typedValue != null)
                {
                    if (this.countryCode != null)
                        this.countryCode.Text = typedValue.CountryCode;
                    if (this.checkDigit != null)
                        this.checkDigit.Text = typedValue.CheckDigit;
                    if (this.CIN != null)
                        this.CIN.Text = typedValue.CIN;
                    if (this.ABI != null)
                        this.ABI.Text = typedValue.ABI;
                    if (this.CAB != null)
                        this.CAB.Text = typedValue.CAB;
                    if (this.accountNumber != null)
                        this.accountNumber.Text = typedValue.AccountNumber;
                }
            }
        }
    }
}
