﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint.MobileControls;
using Microsoft.SharePoint.Utilities;
using Mobile = System.Web.UI.MobileControls;

namespace DevLeap.SP2010.CustomFields
{
    public class IBANMobileFieldControl: SPMobileBaseFieldControl
    {
        private Mobile.TextBox countryCode;
        private Mobile.TextBox checkDigit;
        private Mobile.TextBox CIN;
        private Mobile.TextBox ABI;
        private Mobile.TextBox CAB;
        private Mobile.TextBox accountNumber;

        protected override Mobile.MobileControl CreateControlForDisplay()
        {
            return (this.CreateControlAsLabel());
        }

        protected override Mobile.MobileControl CreateControlForEdit()
        {
            Mobile.MobileControl control = null;
            if (this.Item != null)
            {
                control = this.CreateControlForNew();
            }
            return control;
        }

        protected override Mobile.MobileControl CreateControlForNew()
        {
            if (this.ReadOnly)
            {
                return (this.CreateControlForDisplay());
            }
            else
            {
                return (this.CreateUIControls());
            }
        }

        protected override Mobile.MobileControl CreateControlAsLabel()
        {
            Mobile.MobileControl result = this.CreateControlForView();
            result.BreakAfter = this.BreakAfter;
            return (result);
        }

        protected override Mobile.MobileControl CreateControlForView()
        {
            this.LabelControl.Text = this.Field.GetFieldValueAsText(this.ItemFieldValue).Replace(" ", "-");
            return this.LabelControl;
        }
        
        private Mobile.MobileControl CreateUIControls()
        {
            Mobile.Panel panel = new Mobile.Panel();

            Mobile.LiteralText literalText = new Mobile.LiteralText();
            literalText.BreakAfter = true;
            panel.Controls.Add(literalText);

            Mobile.Label countryCodeLabel = new Mobile.Label();
            countryCodeLabel.Text = "Country Code:";
            panel.Controls.Add(countryCodeLabel);
            this.countryCode = new Mobile.TextBox();
            this.countryCode.MaxLength = 2;
            this.countryCode.BreakAfter = true;
            panel.Controls.Add(this.countryCode);

            Mobile.Label checkDigitLabel = new Mobile.Label();
            checkDigitLabel.Text = "Check Digit:";
            panel.Controls.Add(checkDigitLabel);
            this.checkDigit = new Mobile.TextBox();
            this.checkDigit.MaxLength = 2;
            this.checkDigit.BreakAfter = true;
            panel.Controls.Add(this.checkDigit);

            Mobile.Label CINLabel = new Mobile.Label();
            CINLabel.Text = "CIN:";
            panel.Controls.Add(CINLabel);
            this.CIN = new Mobile.TextBox();
            this.CIN.MaxLength = 1;
            this.CIN.BreakAfter = true;
            panel.Controls.Add(this.CIN);

            Mobile.Label ABILabel = new Mobile.Label();
            ABILabel.Text = "ABI:";
            panel.Controls.Add(ABILabel);
            this.ABI = new Mobile.TextBox();
            this.ABI.MaxLength = 5;
            this.ABI.BreakAfter = true;
            panel.Controls.Add(this.ABI);

            Mobile.Label CABLabel = new Mobile.Label();
            CABLabel.Text = "CAB:";
            panel.Controls.Add(CABLabel);
            this.CAB = new Mobile.TextBox();
            this.CAB.MaxLength = 5;
            this.CAB.BreakAfter = true;
            panel.Controls.Add(this.CAB);

            Mobile.Label AccountNumberLabel = new Mobile.Label();
            AccountNumberLabel.Text = "Account Number:";
            panel.Controls.Add(AccountNumberLabel);
            this.accountNumber = new Mobile.TextBox();
            this.accountNumber.MaxLength = 12;
            this.accountNumber.BreakAfter = true;
            panel.Controls.Add(this.accountNumber);

            this.Controls.Add(panel);

            IBANFieldValue typedValue = this.ItemFieldValue as IBANFieldValue;
            if (typedValue != null)
            {
                if (this.countryCode != null)
                    this.countryCode.Text = typedValue.CountryCode;
                if (this.checkDigit != null)
                    this.checkDigit.Text = typedValue.CheckDigit;
                if (this.CIN != null)
                    this.CIN.Text = typedValue.CIN;
                if (this.ABI != null)
                    this.ABI.Text = typedValue.ABI;
                if (this.CAB != null)
                    this.CAB.Text = typedValue.CAB;
                if (this.accountNumber != null)
                    this.accountNumber.Text = typedValue.AccountNumber;
            }

            return (panel);
        }

        public override object Value
        {
            get
            {
                return new IBANFieldValue
                {
                    CountryCode = this.countryCode.Text,
                    CheckDigit = this.checkDigit.Text,
                    CIN = this.CIN.Text,
                    ABI = this.ABI.Text,
                    CAB = this.CAB.Text,
                    AccountNumber = this.accountNumber.Text,
                };
            }
        }
    }
}
