﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.Office.Word.Server.Conversions;

namespace DevLeap.SP2010.WordConverterConsole
{
    class Program
    {
        static void Main(string[] args)
        {
            Guid jobId = StartConversionJobFromWordToXPS(
                "Shared%20Documents/CH03 - LINQ To Objects.doc");
            MonitorJob(jobId);
        }

        static Guid StartConversionJobFromWordToXPS(String sourceFileUrl)
        {
            String siteUrl = "http://devbook.sp2010.local/";
            String wordAutomationServiceName = "Word Automation Services";

            using (SPSite site = new SPSite(siteUrl))
            {
                using (SPWeb web = site.OpenWeb())
                {
                    SPFile sourceFile = web.GetFile(sourceFileUrl);

                    ConversionJob job = new ConversionJob(wordAutomationServiceName);
                    job.UserToken = site.UserToken;
                    job.Settings.UpdateFields = true;
                    job.Settings.OutputFormat = SaveFormat.XPS;

                    String sourceUrl = web.Url + "/" + sourceFile.Url;
                    String destinationUrl = web.Url + "/" + sourceFile.Url.Replace(".doc", ".xps");
                    job.AddFile(sourceUrl, destinationUrl);
                    job.Start();

                    return (job.JobId);
                }
            }
        }

        static void MonitorJob(Guid jobId)
        { 
            ConversionJobStatus job = new ConversionJobStatus(
                "Word Automation Services", // Name of the service instance
                jobId, // The JobId to monitor
                null); // Guid of the subscription in case of multiple servers

            Console.WriteLine("Job name: {0}", job.Name);
            Console.WriteLine("Total job activities:\t\t{0}", job.Count);
            Console.WriteLine("Job activities succeeded:\t\t{0}", job.Succeeded);
            Console.WriteLine("Job activities failed:\t\t{0}", job.Failed);
            Console.WriteLine("Job activities in progress:\t\t{0}", job.InProgress);
            Console.WriteLine("Job activities canceled:\t\t{0}", job.Canceled);
            Console.WriteLine("Job activities not started:\t\t{0}", job.NotStarted);
        }
    }
}

