using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Administration;

namespace DevLeap.SP2010.ProtocolServiceApp.Features.ProtocolServiceApp
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("9cba5093-a97b-40e3-8a4e-b2272ad0a86a")]
    public class ProtocolServiceAppEventReceiver : SPFeatureReceiver
    {
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            SPFarm farm = SPFarm.Local;
            SPServer server = SPServer.Local;

            ProtocolService service = ProtocolService.CurrentInstance;

            if (service == null)
            {
                service = new ProtocolService(farm);
                service.Update();
            }

            ProtocolServiceInstance serviceInstance = server.ServiceInstances.GetValue<ProtocolServiceInstance>();
            if (serviceInstance == null)
            {
                serviceInstance = new ProtocolServiceInstance(server, service);
                serviceInstance.Update(true);
            }
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            SPFarm farm = SPFarm.Local;
            SPServer server = SPServer.Local;

            ProtocolService service = ProtocolService.CurrentInstance;

            if (service != null)
            {
                ProtocolServiceInstance serviceInstance = server.ServiceInstances.GetValue<ProtocolServiceInstance>();
                if (serviceInstance != null)
                {
                    server.ServiceInstances.Remove(serviceInstance.Id);
                }

                farm.Services.Remove(service.Id);
            }
        }


        // Uncomment the method below to handle the event raised after a feature has been installed.

        //public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised before a feature is uninstalled.

        //public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        //{
        //}

        // Uncomment the method below to handle the event raised when a feature is upgrading.

        //public override void FeatureUpgrading(SPFeatureReceiverProperties properties, string upgradeActionName, System.Collections.Generic.IDictionary<string, string> parameters)
        //{
        //}
    }
}
