﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Management.Automation;
using Microsoft.SharePoint.PowerShell;
using Microsoft.SharePoint.Administration;

namespace DevLeap.SP2010.ProtocolServiceApp.PowerShellRegistration
{
    [Cmdlet(VerbsLifecycle.Install, "ProtocolService", SupportsShouldProcess = true)]
    [SPCmdlet(RequireUserFarmAdmin = true)]
    internal sealed class InstallProtocolService : SPCmdlet
    {
        protected override void InternalProcessRecord()
        {
            // Ensure that the required local farm object is available
            SPFarm farm = SPFarm.Local;
            if (null == farm)
            {
                ThrowTerminatingError(new InvalidOperationException("SharePoint server farm not found."),
                    ErrorCategory.ResourceUnavailable, this);
            }

            // Make sure that the local server is registered in the farm
            SPServer localServer = SPServer.Local;
            if (null == localServer)
            {
                ThrowTerminatingError(new InvalidOperationException("SharePoint local server registration not found."),
                    ErrorCategory.ResourceUnavailable, this);
            }

            if (ShouldProcess("Sample Web Service"))
            {
                // Make sure that the sample web service exists
                ProtocolService service = ProtocolService.CurrentInstance;
                if (null == service)
                {
                    // Create the service
                    service = new ProtocolService(farm);
                    service.Update(true);
                }

                // Make sure that the sample web service instance exists for the local server
                ProtocolServiceInstance serviceInstance = localServer.ServiceInstances.GetValue<ProtocolServiceInstance>();
                if (null == serviceInstance)
                {
                    // Create the service instance
                    serviceInstance = new ProtocolServiceInstance(localServer, service);
                    serviceInstance.Update(true);
                }
            }
        }
    }
}
