﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint.Administration;
using DevLeap.SP2010.ProtocolServiceAppProxy;
using System.ServiceModel.Activation;

namespace DevLeap.SP2010.ProtocolServiceApp
{
    [System.Runtime.InteropServices.Guid("41EE8CFA-D129-436a-A9E3-AB8208271A96")]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public sealed class ProtocolService : SPIisWebService, IServiceAdministration
    {
        #region Constructors

        // Default constructor for serialization purposes
        public ProtocolService()
            : base()
        { }

        // Real constructor based on the SPFarm where we're running
        public ProtocolService(SPFarm farm)
            : base(farm)
        { this.Name = "ProtocolService"; }

        #endregion

        #region Current Instance Singleton

        // Current instance singleton supporting static variables
        private static ProtocolService _currentInstance = null;
        private static Object _currentInstanceSyncLock = new Object();

        // Singleton property to get the CurrentInstance of the ProtocolService
        public static ProtocolService CurrentInstance
        {
            get
            {
                if (_currentInstance == null)
                {
                    lock (_currentInstanceSyncLock)
                    {
                        if (_currentInstance == null)
                        {
                            // Query the SPFarm for the current instance of the ProtocolService
                            _currentInstance = SPFarm.Local.Services.GetValue<ProtocolService>("ProtocolService");
                        }
                    }
                }

                return _currentInstance;
            }
        }

        #endregion

        #region IServiceAdministration

        public SPServiceApplication CreateApplication(string name, Type serviceApplicationType, SPServiceProvisioningContext provisioningContext)
        {
            if (provisioningContext == null)
                throw new ArgumentNullException("provisioningContext");
            if (serviceApplicationType != typeof(ProtocolServiceApplication))
                throw new NotSupportedException("Invalid Service Application type, expected ProtocolService");

            ProtocolServiceApplication application =
                this.Farm.GetObject(
                    name,
                    this.Id,
                    serviceApplicationType) as ProtocolServiceApplication;

            if (application == null)
            {
                SPDatabaseParameters databaseParameters =
                    SPDatabaseParameters.CreateParameters(
                        name,
                        SPDatabaseParameterOptions.GenerateUniqueName);

                application = ProtocolServiceApplication.Create(
                    name,
                    this,
                    provisioningContext.IisWebServiceApplicationPool,
                    databaseParameters);
            }

            return (application);
        }

        public ProtocolServiceApplication CreateApplication(string name, bool checkIfExist, SPIisWebServiceApplicationPool applicationPool)
        {
            if (checkIfExist)
            {
                ProtocolServiceApplication webApplicationByName = this.GetWebApplicationByName(name);
                if (webApplicationByName != null)
                {
                    return webApplicationByName;
                }
            }

            SPDatabaseParameters databaseParameters =
                SPDatabaseParameters.CreateParameters(name, SPDatabaseParameterOptions.GenerateUniqueName);

            ProtocolServiceApplication application = ProtocolServiceApplication.Create(
                name,
                this,
                applicationPool,
                databaseParameters);

            return (application);
        }

        public SPServiceApplicationProxy CreateProxy(string name, SPServiceApplication serviceApplication, SPServiceProvisioningContext provisioningContext)
        {
            if (serviceApplication == null)
                throw new ArgumentNullException("serviceApplication");

            if (serviceApplication.GetType() != typeof(ProtocolServiceApplication))
                throw new NotSupportedException("Invalid SPServiceApplication type");

            ProtocolServiceProxy serviceProxy = (ProtocolServiceProxy)base.Farm.GetObject(
                string.Empty,
                base.Farm.Id,
                typeof(ProtocolServiceProxy));
            if (serviceProxy == null)
            {
                serviceProxy = new ProtocolServiceProxy(base.Farm);
                serviceProxy.Update(true);
            }

            ProtocolServiceApplicationProxy applicationProxy =
                serviceProxy.ApplicationProxies.GetValue<ProtocolServiceApplicationProxy>(name);
            if (applicationProxy == null)
            {
                applicationProxy = new ProtocolServiceApplicationProxy(
                    name,
                    serviceProxy,
                    ((ProtocolServiceApplication)serviceApplication).Uri);
            }

            return applicationProxy;
        }

        public SPPersistedTypeDescription GetApplicationTypeDescription(Type serviceApplicationType)
        {
            if (serviceApplicationType != typeof(ProtocolServiceApplication))
                throw new NotSupportedException("Invalid Service Application type, expected ProtocolService");

            return new SPPersistedTypeDescription("Protocol Service Application", "A custom protocol service");
        }

        public Type[] GetApplicationTypes()
        {
            return (new Type[] { typeof(ProtocolServiceApplication) });
        }

        public override SPAdministrationLink GetCreateApplicationLink(Type serviceApplicationType)
        {
            return new SPAdministrationLink("/_admin/ProtocolService/ProtocolServiceCreate.aspx");
        }

        #endregion

        #region Internal code

        internal ProtocolServiceApplication GetWebApplicationByName(string applicationName)
        {
            return (base.Applications.GetValue<ProtocolServiceApplication>(applicationName));
        }

        #endregion
    }
}
