using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Administration;

namespace DevLeap.SP2010.ProtocolServiceAppProxy.Features.ProtocolServiceAppProxy
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("b65e4fc7-17ba-48eb-b663-52f88dfaf040")]
    public class ProtocolServiceAppProxyEventReceiver : SPFeatureReceiver
    {
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            SPFarm farm = SPFarm.Local;
            SPServer server = SPServer.Local;

            ProtocolServiceProxy serviceProxy = farm.ServiceProxies.GetValue<ProtocolServiceProxy>();

            if (serviceProxy == null)
            {
                serviceProxy = new ProtocolServiceProxy(farm);
                serviceProxy.Update(true);
            }
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            SPFarm farm = SPFarm.Local;
            SPServer server = SPServer.Local;

            ProtocolServiceProxy serviceProxy = farm.ServiceProxies.GetValue<ProtocolServiceProxy>();

            if (serviceProxy != null)
            {
                farm.ServiceProxies.Remove(serviceProxy.Id);
            }
        }


        // Uncomment the method below to handle the event raised after a feature has been installed.

        //public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        //{
        //}


        // Uncomment the method below to handle the event raised before a feature is uninstalled.

        //public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        //{
        //}

        // Uncomment the method below to handle the event raised when a feature is upgrading.

        //public override void FeatureUpgrading(SPFeatureReceiverProperties properties, string upgradeActionName, System.Collections.Generic.IDictionary<string, string> parameters)
        //{
        //}
    }
}
