﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Management.Automation;
using Microsoft.SharePoint.PowerShell;
using Microsoft.SharePoint;
using DevLeap.SP2010.ProtocolServiceAppProxy;

namespace DevLeap.SP2010.ProtocolServiceAppProxy.PowerShellRegistration
{
    [Cmdlet("Invoke", "ProtocolService")]
    internal sealed class InvokeProtocolService : SPCmdlet
    {
        private const string GenerateProtocolNumberSet = "GenerateProtocolNumber";

        [Parameter(Mandatory = true, ValueFromPipeline = true, Position = 0)]
        public SPServiceContextPipeBind ServiceContext { get; set; }

        [Parameter(ParameterSetName = GenerateProtocolNumberSet, Mandatory = true)]
        public String ContentUri { get; set; }

        protected override void InternalProcessRecord()
        {
            SPServiceContext serviceContext = this.ServiceContext.Read();
            if (null == serviceContext)
            {
                WriteError(new InvalidOperationException("Invalid service context."),
                    ErrorCategory.ResourceExists, serviceContext);

                return;
            }

            ProtocolServiceClient client = new ProtocolServiceClient(serviceContext);

            if (this.ParameterSetName == GenerateProtocolNumberSet)
            {
                if (String.IsNullOrEmpty(this.ContentUri))
                {
                    WriteError(new ArgumentNullException("ContentUri"), ErrorCategory.InvalidArgument, this.ContentUri);
                    return;
                }

                ExecuteOptions options = ExecuteOptions.None;

                String result = client.GenerateProtocolNumber(this.ContentUri, options);

                this.WriteObject(result);
            }
            else
            {
                throw new NotSupportedException("Parameter set not supported.");
            }
        }
    }
}
