﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.Workflow;
using Microsoft.SharePoint.Utilities;

namespace DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow
{
    public partial class OffersApprovalInitiationForm : LayoutsPageBase
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            InitializeParams();

            // Optionally, add code here to pre-populate your form fields.
            if (!this.IsPostBack)
            {
                this.associationGuid = Request.Params["TemplateID"];

                // Retrieve the current AssociationData configuration
                SPWorkflowAssociation association = this.workflowList.WorkflowAssociations[new Guid(this.associationGuid)];
                if ((association != null) && !String.IsNullOrEmpty(association.AssociationData))
                {
                    AssociationDataConfiguration associationData = WorkflowUtility.DeserializeData<AssociationDataConfiguration>(association.AssociationData);

                    // Set the current value for the PeopleEditor control
                    this.OfferManager.CommaSeparatedAccounts = associationData.OfferManager;
                    // And validate that value
                    this.OfferManager.Validate();
                }
            }
        }

        // This method is called when the user clicks the button to start the workflow.
        private string GetInitiationData()
        {
            // Define an instance of the serializable type holding the InitiationData
            InitiationDataConfiguration initiationData = new InitiationDataConfiguration();

            // Validate the content of the PeopleEditor in order to populate the list of Entities
            this.OfferManager.Validate();

            // If there are selected entities
            if (this.OfferManager.Entities.Count > 0)
            {
                // Set the currently selected value for the OfferManager field
                initiationData.OfferManager = ((PickerEntity)this.OfferManager.Entities[0]).Description;
            }

            // Set the value for the ApprovalRequestNotes field
            initiationData.ApprovalRequestNotes = this.ApprovalRequestNotes.Text;

            // Return the InitiationData serialized as an XML String
            return WorkflowUtility.SerializeData(initiationData);
        }

        protected void StartWorkflow_Click(object sender, EventArgs e)
        {
            // Optionally, add code here to perform additional steps before starting your workflow
            try
            {
                HandleStartWorkflow();
            }
            catch (Exception)
            {
                SPUtility.TransferToErrorPage(SPHttpUtility.UrlKeyValueEncode("Failed to Start Workflow"));
            }
        }

        protected void Cancel_Click(object sender, EventArgs e)
        {
            SPUtility.Redirect("Workflow.aspx", SPRedirectFlags.RelativeToLayoutsPage, HttpContext.Current, Page.ClientQueryString);
        }

        #region Workflow Initiation Code - Typically, the following code should not be changed

        private string associationGuid;
        private SPList workflowList;
        private SPListItem workflowListItem;

        private void InitializeParams()
        {
            try
            {
                this.associationGuid = Request.Params["TemplateID"];

                // Parameters 'List' and 'ID' will be null for site workflows.
                if (!String.IsNullOrEmpty(Request.Params["List"]) && !String.IsNullOrEmpty(Request.Params["ID"]))
                {
                    this.workflowList = this.Web.Lists[new Guid(Request.Params["List"])];
                    this.workflowListItem = this.workflowList.GetItemById(Convert.ToInt32(Request.Params["ID"]));
                }
            }
            catch (Exception)
            {
                SPUtility.TransferToErrorPage(SPHttpUtility.UrlKeyValueEncode("Failed to read Request Parameters"));
            }
        }

        private void HandleStartWorkflow()
        {
            if (this.workflowList != null && this.workflowListItem != null)
            {
                StartListWorkflow();
            }
            else
            {
                StartSiteWorkflow();
            }
        }

        private void StartSiteWorkflow()
        {
            SPWorkflowAssociation association = this.Web.WorkflowAssociations[new Guid(this.associationGuid)];
            this.Web.Site.WorkflowManager.StartWorkflow((object)null, association, GetInitiationData(), SPWorkflowRunOptions.Synchronous);
            SPUtility.Redirect(this.Web.Url, SPRedirectFlags.UseSource, HttpContext.Current);
        }

        private void StartListWorkflow()
        {
            SPWorkflowAssociation association = this.workflowList.WorkflowAssociations[new Guid(this.associationGuid)];
            this.Web.Site.WorkflowManager.StartWorkflow(workflowListItem, association, GetInitiationData());
            SPUtility.Redirect(this.workflowList.DefaultViewUrl, SPRedirectFlags.UseSource, HttpContext.Current);
        }
        #endregion
    }
}