﻿using System;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Collections;
using System.Drawing;
using System.Linq;
using System.Workflow.ComponentModel.Compiler;
using System.Workflow.ComponentModel.Serialization;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Design;
using System.Workflow.Runtime;
using System.Workflow.Activities;
using System.Workflow.Activities.Rules;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Workflow;
using Microsoft.SharePoint.WorkflowActions;

namespace DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow
{
    public sealed partial class OffersApprovalWorkflow : SequentialWorkflowActivity
    {
        public OffersApprovalWorkflow()
        {
            InitializeComponent();
        }

        public Guid workflowId = default(System.Guid);
        public SPWorkflowActivationProperties workflowProperties = new SPWorkflowActivationProperties();

        public AssociationDataConfiguration AssociationData;
        public InitiationDataConfiguration InitiationData;

        private void onWorkflowActivated_Invoked(object sender, ExternalDataEventArgs e)
        {
            // Retrieve the current AssociationData configuration
            this.AssociationData = WorkflowUtility.DeserializeData<AssociationDataConfiguration>(this.workflowProperties.AssociationData);
            // Check if there is also the InitiationData configuration
            if (!String.IsNullOrEmpty(this.workflowProperties.InitiationData))
                this.InitiationData = WorkflowUtility.DeserializeData<InitiationDataConfiguration>(this.workflowProperties.InitiationData);
        }

        private void checkOfferEuroAmount(object sender, ConditionalEventArgs e)
        {
            // Extract the OfferEuroAmount field from the target of invokation
            Guid offerEuroAmountGuid = new Guid("54C4A2FF-D9F4-495F-B18B-AFB32A58F78A");
            Decimal offerEuroAmount = (Decimal)((Double)this.workflowProperties.Item[offerEuroAmountGuid]);

            // Check if the OfferEuroAmount exceeds the limit
            e.Result = (offerEuroAmount >= this.AssociationData.OfferEuroAmountLimit);
        }

        public static DependencyProperty ApprovalTaskIdProperty = DependencyProperty.Register("ApprovalTaskId", typeof(System.Guid), typeof(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow));

        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)]
        [BrowsableAttribute(true)]
        [CategoryAttribute("Misc")]
        public Guid ApprovalTaskId
        {
            get
            {
                return ((System.Guid)(base.GetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskIdProperty)));
            }
            set
            {
                base.SetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskIdProperty, value);
            }
        }

        public static DependencyProperty ApprovalTaskPropertiesProperty = DependencyProperty.Register("ApprovalTaskProperties", typeof(Microsoft.SharePoint.Workflow.SPWorkflowTaskProperties), typeof(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow));

        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)]
        [BrowsableAttribute(true)]
        [CategoryAttribute("Misc")]
        public SPWorkflowTaskProperties ApprovalTaskProperties
        {
            get
            {
                return ((Microsoft.SharePoint.Workflow.SPWorkflowTaskProperties)(base.GetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskPropertiesProperty)));
            }
            set
            {
                base.SetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskPropertiesProperty, value);
            }
        }

        public static DependencyProperty ApprovalTaskAfterPropertiesProperty = DependencyProperty.Register("ApprovalTaskAfterProperties", typeof(Microsoft.SharePoint.Workflow.SPWorkflowTaskProperties), typeof(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow));

        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)]
        [BrowsableAttribute(true)]
        [CategoryAttribute("Misc")]
        public SPWorkflowTaskProperties ApprovalTaskAfterProperties
        {
            get
            {
                return ((Microsoft.SharePoint.Workflow.SPWorkflowTaskProperties)(base.GetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskAfterPropertiesProperty)));
            }
            set
            {
                base.SetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskAfterPropertiesProperty, value);
            }
        }

        public static DependencyProperty ApprovalTaskBeforePropertiesProperty = DependencyProperty.Register("ApprovalTaskBeforeProperties", typeof(Microsoft.SharePoint.Workflow.SPWorkflowTaskProperties), typeof(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow));

        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)]
        [BrowsableAttribute(true)]
        [CategoryAttribute("Misc")]
        public SPWorkflowTaskProperties ApprovalTaskBeforeProperties
        {
            get
            {
                return ((Microsoft.SharePoint.Workflow.SPWorkflowTaskProperties)(base.GetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskBeforePropertiesProperty)));
            }
            set
            {
                base.SetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskBeforePropertiesProperty, value);
            }
        }

        private void whileApprovalTaskNotCompleted(object sender, ConditionalEventArgs e)
        {
            // This method is invoked for every loop of the While activity
            e.Result = !taskCompleted;
        }

        private void createApprovalTask_MethodInvoking(object sender, EventArgs e)
        {
            // Assign a unique ID to the ApprovalTask
            this.ApprovalTaskId = Guid.NewGuid();

            // Configure the properties of the ApprovalTask
            this.ApprovalTaskProperties = new SPWorkflowTaskProperties();

            this.ApprovalTaskProperties.AssignedTo = ((this.InitiationData != null) &&
                !String.IsNullOrEmpty(this.InitiationData.OfferManager)) ?
                this.InitiationData.OfferManager : this.AssociationData.OfferManager;
            if ((this.InitiationData != null) &&
                !String.IsNullOrEmpty(this.InitiationData.ApprovalRequestNotes))
            {
                this.ApprovalTaskProperties.Description = this.InitiationData.ApprovalRequestNotes;
            }
            else
            {
                this.ApprovalTaskProperties.Description = "Please approve the current offer";
            }

            this.ApprovalTaskProperties.DueDate = DateTime.Now.AddDays(10);
            this.ApprovalTaskProperties.SendEmailNotification = true;
            this.ApprovalTaskProperties.Title = String.Format("Please approve offer {0}", this.workflowProperties.Item.Title);
        }

        public Boolean taskCompleted = false;

        private void onApprovalTaskChanged_Invoked(object sender, ExternalDataEventArgs e)
        {
            Guid taskOutcomeGuid = new Guid("841410DE-EFEB-49B3-8A8E-0D006FFCE879");
            this.ApprovalTaskOutcome = this.ApprovalTaskAfterProperties.ExtendedProperties[taskOutcomeGuid].ToString();
            if (this.ApprovalTaskOutcome.Equals("Approved") || this.ApprovalTaskOutcome.Equals("Rejected"))
                this.taskCompleted = true;
            else
                this.taskCompleted = false;
        }

        public static DependencyProperty ApprovalTaskOutcomeProperty = DependencyProperty.Register("ApprovalTaskOutcome", typeof(System.String), typeof(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow));

        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Visible)]
        [BrowsableAttribute(true)]
        [CategoryAttribute("Misc")]
        public String ApprovalTaskOutcome
        {
            get
            {
                return ((string)(base.GetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskOutcomeProperty)));
            }
            set
            {
                base.SetValue(DevLeap.SP2010.FormsApprovalWorkflow.OffersApprovalWorkflow.OffersApprovalWorkflow.ApprovalTaskOutcomeProperty, value);
            }
        }
    }
}
