﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Workflow.ComponentModel;
using System.ComponentModel.Design;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WorkflowActions;
using Microsoft.SharePoint.Utilities;

namespace DevLeap.SP2010.Activities
{
    public class AssignUserToGroup : Activity
    {
        public static DependencyProperty UserNameProperty = DependencyProperty.Register("UserName", typeof(String), typeof(AssignUserToGroup));

        [System.ComponentModel.Description("UserName")]
        [System.ComponentModel.Category("Custom Properties")]
        [System.ComponentModel.Browsable(true)]
        [System.ComponentModel.DesignerSerializationVisibility(System.ComponentModel.DesignerSerializationVisibility.Visible)]
        public String UserName
        {
            get
            {
                return ((String)(base.GetValue(AssignUserToGroup.UserNameProperty)));
            }
            set
            {
                base.SetValue(AssignUserToGroup.UserNameProperty, value);
            }
        }

        public static DependencyProperty GroupProperty = DependencyProperty.Register("Group", typeof(String), typeof(AssignUserToGroup));

        [System.ComponentModel.Description("Group")]
        [System.ComponentModel.Category("Custom Properties")]
        [System.ComponentModel.Browsable(true)]
        [System.ComponentModel.DesignerSerializationVisibility(System.ComponentModel.DesignerSerializationVisibility.Visible)]
        public String Group
        {
            get
            {
                return ((String)(base.GetValue(AssignUserToGroup.GroupProperty)));
            }
            set
            {
                base.SetValue(AssignUserToGroup.GroupProperty, value);
            }
        }

        public static DependencyProperty __ContextProperty = DependencyProperty.Register("__Context", typeof(WorkflowContext), typeof(AssignUserToGroup));

        [System.ComponentModel.Description("__Context")]
        [System.ComponentModel.Category("__Context Category")]
        [System.ComponentModel.Browsable(true)]
        [System.ComponentModel.DesignerSerializationVisibility(System.ComponentModel.DesignerSerializationVisibility.Visible)]
        public WorkflowContext __Context
        {
            get
            {
                return ((WorkflowContext)(base.GetValue(AssignUserToGroup.__ContextProperty)));
            }
            set
            {
                base.SetValue(AssignUserToGroup.__ContextProperty, value);
            }
        }

        protected override ActivityExecutionStatus Execute(ActivityExecutionContext executionContext)
        {
            // Execute the code with elevated privileges
            SPSecurity.RunWithElevatedPrivileges(
                delegate()
                {
                    // Get a reference to the current SPSite
                    using (SPSite site = new SPSite(this.__Context.Site.ID))
                    {
                        // Open the current SPWeb
                        using (SPWeb web = site.OpenWeb(this.__Context.Web.ID))
                        {
                            // Ensure the user
                            SPUser user = web.EnsureUser(this.UserName);

                            // Add him to the target group
                            web.Groups[this.Group].AddUser(user);
                        }
                    }
                });
            
            return (ActivityExecutionStatus.Closed);
        }
    }
}
