﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint.Workflow;
using System.Threading;
using Microsoft.SharePoint;
using System.Workflow.Runtime;

namespace DevLeap.SP2010.ExternalCRMService
{
    class WorkflowContextState
    {
        public SPWeb Web { get; set; }
        public Guid InstanceId { get; set; }
    }

    public class CRMService : SPWorkflowExternalDataExchangeService, ICRMService
    {
        public void SubmitOffer(Offer item)
        {
            // Simulate an external CRM engine
            ThreadPool.QueueUserWorkItem(delegate(Object state)
            {
                WorkflowContextState workflowState = state as WorkflowContextState;

                if (workflowState != null)
                {
                    // Simulate a random workload
                    Random rnd = new Random();
                    Thread.Sleep(TimeSpan.FromSeconds(rnd.Next(1, 60)));

                    RaiseEvent(
                        workflowState.Web,
                        workflowState.InstanceId,
                        typeof(ICRMService),
                        "CRMCodeAvailable",
                        new Object[] { String.Format("CRM_CODE_{0:0000}", rnd.Next(1, 1000)) });
                }
            }, new WorkflowContextState { 
                Web = this.CurrentWorkflow.ParentWeb,
                InstanceId = WorkflowEnvironment.WorkflowInstanceId
            });
        }

        public event EventHandler<CRMCodeEventArgs> CRMCodeAvailable;

        public override void CallEventHandler(Type eventType, string eventName, 
            object[] eventData, SPWorkflow workflow, string identity, 
            System.Workflow.Runtime.IPendingWork workHandler, object workItem)
        {
            // Prepare the custom ExternalDataEventArgs instance
            CRMCodeEventArgs args = new CRMCodeEventArgs(workflow.InstanceId)
            { 
                WorkHandler = workHandler,
                WorkItem = workItem,
                Identity = identity,
                // Get the result from the Object[] with name EventData
                CRMCode = eventData[0].ToString(),
            };

            this.OnOfferApprovalOutcomeAvailable(args);
        }

        protected virtual void OnOfferApprovalOutcomeAvailable(CRMCodeEventArgs args)
        {
            if (this.CRMCodeAvailable != null)
            {
                // Pass null as the event source to avoid having 
                // serialization issues during workflow persistence
                this.CRMCodeAvailable(null, args);
            }
        }

        public override void CreateSubscription(System.Workflow.Activities.MessageEventSubscription subscription)
        {
            throw new NotImplementedException();
        }

        public override void DeleteSubscription(Guid subscriptionId)
        {
            throw new NotImplementedException();
        }
    }
}
