using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Administration;
using System.Linq;

namespace DevLeap.SP2010.ExternalCRMServicePackage.Features.CRMServiceFeature
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("1d5a5acc-e9bc-45ab-8b7f-d6b4f1d82ffa")]
    public class CRMServiceFeatureEventReceiver : SPFeatureReceiver
    {
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            // The parent of this feature receiver is an SPWebApplication
            SPWebApplication webApplication = properties.Feature.Parent as SPWebApplication;

            if (webApplication != null)
            {
                // Define the web.config modification
                SPWebConfigModification modification =
                            new SPWebConfigModification()
                            {
                                Name = "CRMWorkflowService",
                                Path = "configuration/SharePoint/WorkflowServices",
                                Owner = "DevLeap",
                                Sequence = 0,
                                Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode,
                                Value = "<WorkflowService Assembly=\"DevLeap.SP2010.ExternalCRMService, Version=1.0.0.0, Culture=neutral, PublicKeyToken=665aaca1c91d20cb\" Class=\"DevLeap.SP2010.ExternalCRMService.CRMService\"></WorkflowService>"
                            };

                // Apply it to the web.config of the current SPWebApplication
                webApplication.WebConfigModifications.Add(modification);
                webApplication.Update();
                webApplication.WebService.ApplyWebConfigModifications();
            }
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            // The parent of this feature receiver is an SPWebApplication
            SPWebApplication webApplication = properties.Feature.Parent as SPWebApplication;

            if (webApplication != null)
            {
                // Search the modification to remove
                var modification =
                    (from m in webApplication.WebConfigModifications
                     where m.Name == "CRMWorkflowService"
                     select m).FirstOrDefault();

                // If any
                if (modification != null)
                {
                    // Remove it to the web.config of the current SPWebApplication
                    webApplication.WebConfigModifications.Remove(modification);
                    webApplication.Update();
                    webApplication.WebService.ApplyWebConfigModifications();
                }
            }
        }
    }
}
