﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

namespace DevLeap.SP2010.WorkflowSOAP
{
    class Program
    {
        static void Main(string[] args)
        {
            // Configure the service proxy
            WorkflowService.Workflow wfSoapService =
                new WorkflowService.Workflow();
            wfSoapService.Credentials = System.Net.CredentialCache.DefaultCredentials;

            // Configure the URL of the target specific item
            String itemUrl = "http://devbook.sp2010.local/DevLeap%20CRM%20Offers/Offer-01.docx";

            // Retrieve the workflow templates associated with the target item
            XmlNode templates = wfSoapService.GetTemplatesForItem(itemUrl);

            // Retrieve the templateId of the workflow to start
            XmlNamespaceManager ns = new XmlNamespaceManager(templates.OwnerDocument.NameTable);
            ns.AddNamespace("wf", "http://schemas.microsoft.com/sharepoint/soap/workflow/");
            XmlNode templateIdNode = templates.SelectSingleNode("//wf:WorkflowTemplate[@Name = 'CRMOffersApprovalWorkflow']/wf:WorkflowTemplateIdSet/@TemplateId", ns);
            Guid templateId = new Guid(templateIdNode.Value);

            XmlDocument initiationData = new XmlDocument();
            initiationData.LoadXml("<?xml version=\"1.0\" encoding=\"utf-16\"?><CRMInitiationDataConfiguration xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns=\"http://schemas.devleap.com/CRMInitiationDataConfiguration\"><OfferManager>SP2010DEV\\Administrator</OfferManager><ApprovalRequestNotes>Please approve this offer</ApprovalRequestNotes></CRMInitiationDataConfiguration>");

            XmlNode startResult = wfSoapService.StartWorkflow(itemUrl, templateId, initiationData);

            XmlNode toDosForItem = wfSoapService.GetToDosForItem(itemUrl);
            Console.WriteLine(toDosForItem.OuterXml);

            XmlNode wfDataForItem = wfSoapService.GetWorkflowDataForItem(itemUrl);
            Console.WriteLine(wfDataForItem.OuterXml);

            Guid taskListID = new Guid("{50C9BDC6-BD15-4167-9514-5E6E873A3908}");
            Int32 taskItemID = 30;

            XmlNode taskDataResult = wfSoapService.GetWorkflowTaskData(itemUrl, taskItemID, taskListID);
            Console.WriteLine(taskDataResult.OuterXml);

            XmlDocument taskData = new XmlDocument();
            taskData.LoadXml("<fields/>");
            taskData.DocumentElement.AppendChild(taskData.CreateElement("DevLeapCRMApprovalTaskNotes"));
            taskData.DocumentElement.LastChild.InnerText = "Task approved by remote smart client!";
            taskData.DocumentElement.AppendChild(taskData.CreateElement("DevLeapCRMApprovalTaskOutcome"));
            taskData.DocumentElement.LastChild.InnerText = "Approved";
            taskData.DocumentElement.AppendChild(taskData.CreateElement("Status"));
            taskData.DocumentElement.LastChild.InnerText = "Completed";
            XmlNode alterToDoResult = wfSoapService.AlterToDo(itemUrl, taskItemID, taskListID, taskData);
            Console.WriteLine(alterToDoResult.OuterXml);
        }
    }
}
