﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Workflow;
using DevLeap.SP2010.CRMOffersApproval;
using System.Collections;

namespace DevLeap.SP2010.WorkflowSOM
{
    class Program
    {
        static void Main(string[] args)
        {
            // StartWorkflowOnAllItems();
            // GetRunningWorkflowsOnItem();
            // GetPendingTaskForRunningWorkflowOnItem();
            CompletePendingTaskForRunningWorkflowOnItem();
        }

        // Listing 20-11
        static void StartWorkflowOnAllItems()
        {
            // Retrieve e reference to the target SPSite and SPWeb
            SPSite site = new SPSite("http://devbook.sp2010.local/");
            SPWeb web = site.OpenWeb();

            // Retrieve e reference to the target library
            SPList listOffers = web.Lists["DevLeap CRM Offers"];

            // Retrieve a reference to the workflow association to start
            var wfAssociation =
                (from SPWorkflowAssociation wfa in listOffers.WorkflowAssociations
                 where wfa.Name == "CRMOffersApprovalWorkflow"
                 select wfa).FirstOrDefault();

            // If any
            if (wfAssociation != null)
            {
                // Prepare Initiation Data
                CRMInitiationDataConfiguration initData =
                    new CRMInitiationDataConfiguration
                    {
                        OfferManager = "SP2010DEV\\Administrator",
                        ApprovalRequestNotes = "Please approve this offer",
                    };

                String initDataText = WorkflowUtility.SerializeData<CRMInitiationDataConfiguration>(initData);

                // For each item in the list, start the workflow
                // unless it is not already running
                foreach (SPListItem item in listOffers.Items)
                {
                    var workflowAlreadyRunning =
                        (from SPWorkflow wf in item.Workflows
                         where wf.AssociationId == wfAssociation.Id
                         && wf.InternalState == SPWorkflowState.Running
                         select wf).Count();

                    if (workflowAlreadyRunning == 0)
                    {
                        // Start a new workflow instance
                        SPWorkflow wf = site.WorkflowManager.StartWorkflow(
                        item, wfAssociation, initDataText);
                        Guid wfInstanceId = wf.InstanceId;

                        Console.WriteLine("Started instance {0} on item {1}",
                            wfInstanceId, item.Title);
                    }
                }
            }
        }

        // Listing 20-12
        static void GetRunningWorkflowsOnItem()
        {
            // Retrieve e reference to the target SPSite and SPWeb
            SPSite site = new SPSite("http://devbook.sp2010.local/");
            SPWeb web = site.OpenWeb();

            // Retrieve e reference to the target library
            SPList listOffers = web.Lists["DevLeap CRM Offers"];
            SPListItem item = listOffers.Items[0];

            SPWorkflowCollection runningWorkflows = 
                site.WorkflowManager.GetItemActiveWorkflows(
                item);

            Console.WriteLine("There are {0} workflow instances running on item {1}",
                runningWorkflows.Count, item.Title);
        }

        // Listing 20-13
        static void GetPendingTaskForRunningWorkflowOnItem()
        {
            // Retrieve e reference to the target SPSite and SPWeb
            SPSite site = new SPSite("http://devbook.sp2010.local/");
            SPWeb web = site.OpenWeb();

            // Retrieve e reference to the target library
            SPList listOffers = web.Lists["DevLeap CRM Offers"];
            SPListItem item = listOffers.Items[0];

            // Retrieve the running workflow instances
            SPWorkflowCollection runningWorkflows = 
                site.WorkflowManager.GetItemActiveWorkflows(
                item);

            // Get the first running instance
            SPWorkflow runningInstance = runningWorkflows[0];

            // Define a filter to retrieve only pending tasks
            SPWorkflowFilter filter = new SPWorkflowFilter(
                SPWorkflowState.Running,
                SPWorkflowState.None,
                SPWorkflowAssignedToFilter.None);

            // Retrieve pending tasks
            SPWorkflowTaskCollection pendingTasks =
                site.WorkflowManager.GetWorkflowTasks(
                item, runningInstance.InstanceId, filter);

            Console.WriteLine("There are {0} pending tasks running on item {1} for workflow instance {2}",
                pendingTasks.Count, item.Title, runningInstance.InstanceId);
        }

        // Listing 20-14
        static void CompletePendingTaskForRunningWorkflowOnItem()
        {
            // Retrieve e reference to the target SPSite and SPWeb
            SPSite site = new SPSite("http://devbook.sp2010.local/");
            SPWeb web = site.OpenWeb();

            // Retrieve e reference to the target library
            SPList listOffers = web.Lists["DevLeap CRM Offers"];
            SPListItem item = listOffers.Items[0];

            // Retrieve the running workflow instances
            SPWorkflowCollection runningWorkflows =
                site.WorkflowManager.GetItemActiveWorkflows(
                item);

            // Get the first running instance
            SPWorkflow runningInstance = runningWorkflows[0];

            // Define a filter to retrieve only pending tasks
            SPWorkflowFilter filter = new SPWorkflowFilter(
                SPWorkflowState.Running,
                SPWorkflowState.None,
                SPWorkflowAssignedToFilter.None);

            // Retrieve pending tasks
            SPWorkflowTaskCollection pendingTasks =
                site.WorkflowManager.GetWorkflowTasks(
                item, runningInstance.InstanceId, filter);

            if (pendingTasks.Count == 1)
            {
                // Get the target task, if any
                SPWorkflowTask targetTask = pendingTasks[0];

                // Complete the task
                Hashtable taskProperties = new Hashtable();
                taskProperties[FieldsIds.DevLeapCRMApprovalTaskStatus_Id] = "Completed";
                taskProperties[FieldsIds.DevLeapCRMApprovalTaskNotes_Id] = "Automatically approved!";
                taskProperties[FieldsIds.DevLeapCRMApprovalTaskOutcome_Id] = "Approved";

                SPWorkflowTask.AlterTask(targetTask, taskProperties, true);

                Console.WriteLine("Pending task automatically completed!");
            }
        }
    }
}
