﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.SharePoint.UserCode;
using Microsoft.SharePoint.Administration;

namespace DevLeap.SP2010.SandboxValidator
{
    [Guid("39C408AE-AE75-4FFC-BBC2-D420A0207981")]
    public class DevLeapSolutionValidator : SPSolutionValidator
    {
        private const string validatorName = "DevLeap Solution Validator";

        public DevLeapSolutionValidator()
        { }

        public DevLeapSolutionValidator(SPUserCodeService userCodeService) :
            base(validatorName, userCodeService)
        {
            // This should be an hash code of the validator solution
            // For the sake of simplicity it has been used 
            // the current year value
            this.Signature = 2011;
        }

        public override void ValidateAssembly(SPSolutionValidationProperties properties, SPSolutionFile assembly)
        {
            base.ValidateAssembly(properties, assembly);

            // Check if the assembly name contains DevLeap
            if (assembly.Location.Contains("DevLeap"))
            {
                // Set the Valid flag to false
                properties.Valid = false;
                // Set the error message
                properties.ValidationErrorMessage = "Invalid assembly file";
                // Redirect to a custom error page (application page)
                properties.ValidationErrorUrl = String.Format(
                    "/_layouts/DevLeap.SP2010.SandboxValidator/InvalidAssembly.aspx?package={0}&assembly={1}",
                    properties.PackageFile.Location, assembly.Location);
            }
            else
            {
                // Set the Valid flag to true
                properties.Valid = true;
            }
        }

        public override void ValidateSolution(SPSolutionValidationProperties properties)
        {
            base.ValidateSolution(properties);

            // Check if the package file name contains DevLeap
            if (properties.PackageFile.Location.Contains("DevLeap"))
            {
                // Set the Valid flag to false
                properties.Valid = false;
                // Set the error message
                properties.ValidationErrorMessage = "Invalid solution package";
            }
            else
            {
                foreach (SPSolutionFile file in properties.Files)
                {
                    // You could also check every single file of the solution
                }
            }

            // Set the Valid flag to true
            properties.Valid = true;
        }
    }
}
