using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.UserCode;

namespace DevLeap.SP2010.FullTrustProxy.Features.FullTrustProxyFeature
{
    /// <summary>
    /// This class handles events raised during feature activation, deactivation, installation, uninstallation, and upgrade.
    /// </summary>
    /// <remarks>
    /// The GUID attached to this class may be used during packaging and should not be modified.
    /// </remarks>

    [Guid("5a1b6767-dc11-40e0-8785-0be2271b2bd1")]
    public class FullTrustProxyFeatureEventReceiver : SPFeatureReceiver
    {
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            // Retrieve e reference to the UserCodeService
            SPUserCodeService userCodeService = SPUserCodeService.Local;
            if (userCodeService != null)
            {
                // Define a variable to describe the proxy
                SPProxyOperationType proxyOperation =
                    new SPProxyOperationType(
                        this.GetType().Assembly.FullName,
                        typeof(RetrieveNWindCustomers).FullName);

                // Add the proxy to the UserCodeService
                userCodeService.ProxyOperationTypes.Add(proxyOperation);
                
                // Save changes
                userCodeService.Update();
            }
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            // Retrieve e reference to the UserCodeService
            SPUserCodeService userCodeService = SPUserCodeService.Local;
            if (userCodeService != null)
            {
                // Define a variable to describe the proxy
                SPProxyOperationType proxyOperation =
                    new SPProxyOperationType(
                        this.GetType().Assembly.FullName,
                        typeof(RetrieveNWindCustomers).FullName);

                // Remove the proxy to the UserCodeService
                userCodeService.ProxyOperationTypes.Remove(proxyOperation);

                // Save changes
                userCodeService.Update();
            }
        }
    }
}
