﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Office.Server.Search.Query;
using Microsoft.Office.Server.Search.Administration;
using Microsoft.SharePoint;
using System.Xml;
using Microsoft.Office.Server;
using System.Data;

namespace DevLeap.SP2010.SearchProgramming
{
    class Program
    {
        static void Main(string[] args)
        {
            // BrowseLocations("http://devbook.sp2010.local/");
            // SearchByFederation("http://devbook.sp2010.local/", "SOAP");
            // SearchByQueryObjectModelKeyword("http://devbook.sp2010.local/", "SOAP");
            // SearchByQueryObjectModelSQL("http://devbook.sp2010.local/", "SOAP");
        }

        static void BrowseLocations(String siteUrl)
        {
            using (SPSite site = new SPSite(siteUrl))
            {
                Int64 lastupdate;
                Boolean useCrawlProxy;
                // Retrieve a reference to the search service application proxy
                SearchServiceApplicationProxy searchServiceProxy =
                    (SearchServiceApplicationProxy)SearchServiceApplicationProxy.GetProxy(SPServiceContext.GetContext(site));

                // Retrieve currently configured locations
                LocationConfiguration[] locationConfigurations =
                    searchServiceProxy.GetLocationConfigurations(out lastupdate, out useCrawlProxy);

                // Browse locations
                foreach (LocationConfiguration lc in locationConfigurations)
                {
                    Console.WriteLine("***************************************");
                    Console.WriteLine("Location: {0}", lc.InternalName);
                    Console.WriteLine("Type: {0}", lc.Type);
                    Console.WriteLine("URL Template: {0}", lc.ConnectionUrlTemplate);
                }
            }
        }

        static void SearchByFederation(String siteUrl, String searchTerms)
        {
            using (SPSite site = new SPSite(siteUrl))
            {
                Int64 lastupdate;
                Boolean useCrawlProxy;
                // Retrieve a reference to the search service application proxy
                SearchServiceApplicationProxy searchServiceProxy =
                    (SearchServiceApplicationProxy)SearchServiceApplicationProxy.GetProxy(SPServiceContext.GetContext(site));

                // Retrieve currently configured locations
                LocationConfiguration[] locationConfigurations =
                    searchServiceProxy.GetLocationConfigurations(out lastupdate, out useCrawlProxy);

                LocationList locations = new LocationList();

                // Prepare locations for search
                foreach (LocationConfiguration lc in locationConfigurations)
                {
                    locations.Add(new Location(lc.InternalName, searchServiceProxy));
                    // locations[locations.Count - 1].UserQuery = searchTerms;
                }

                // Create a QueryManager instance
                QueryManager manager = new QueryManager();
                manager.Add(locations);
                manager.UserQuery = searchTerms;

                // Search across all locations
                XmlDocument results = manager.GetResults(locations);
                Console.WriteLine(results.OuterXml);

                results.Save(@"..\..\SearchOutput.xml");
            }
        }

        static void SearchByQueryObjectModelKeyword(String siteUrl, String searchTerms)
        {
            using (SPSite site = new SPSite(siteUrl))
            {
                // Retrieve a reference to the search service application proxy
                SearchServiceApplicationProxy searchServiceProxy =
                    (SearchServiceApplicationProxy)SearchServiceApplicationProxy.GetProxy(SPServiceContext.GetContext(site));
                KeywordQuery keywordQuery = new KeywordQuery(searchServiceProxy);

                // Define the query type
                keywordQuery.ResultTypes = ResultType.RelevantResults;
                keywordQuery.QueryText = searchTerms;

                // Execute the query
                ResultTableCollection result = keywordQuery.Execute();

                // Browse the results, if any
                if (result.Count > 0)
                {
                    // Convert results into a DataTable
                    ResultTable relevantResults = result[ResultType.RelevantResults];
                    DataTable resultsDataTable = new DataTable();
                    resultsDataTable.Load(relevantResults, LoadOption.OverwriteChanges);

                    Console.WriteLine("Number of results: {0}", resultsDataTable.Rows.Count);

                    foreach (DataRow row in resultsDataTable.Rows)
                    {
                        Console.WriteLine(row["Title"]);
                    }
                }
            }
        }

        static void SearchByQueryObjectModelSQL(String siteUrl, String searchTerms)
        {
            using (SPSite site = new SPSite(siteUrl))
            {
                // Retrieve a reference to the search service application proxy
                SearchServiceApplicationProxy searchServiceProxy =
                    (SearchServiceApplicationProxy)SearchServiceApplicationProxy.GetProxy(SPServiceContext.GetContext(site));
                FullTextSqlQuery sqlQuery = new FullTextSqlQuery(searchServiceProxy);

                // Define the query type
                sqlQuery.ResultTypes = ResultType.RelevantResults;
                sqlQuery.QueryText = String.Format("SELECT Title, Path FROM Scope() WHERE FREETEXT('{0}')", searchTerms);

                // Execute the query
                ResultTableCollection result = sqlQuery.Execute();

                // Browse the results, if any
                if (result.Count > 0)
                {
                    // Convert results into a DataTable
                    ResultTable relevantResults = result[ResultType.RelevantResults];
                    DataTable resultsDataTable = new DataTable();
                    resultsDataTable.Load(relevantResults, LoadOption.OverwriteChanges);

                    Console.WriteLine("Number of results: {0}", resultsDataTable.Rows.Count);

                    foreach (DataRow row in resultsDataTable.Rows)
                    {
                        Console.WriteLine("{0}\t{1}", row["Title"], row["Path"]);
                    }
                }
            }
        }
    }
}
