﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.BusinessData.SharedService;
using Microsoft.SharePoint.BusinessData.MetadataModel;
using Microsoft.BusinessData.MetadataModel;
using Microsoft.BusinessData.Runtime;

namespace DevLeap.SP2010.BCSConsumer
{
    class Program
    {
        static void Main(string[] args)
        {
            using (SPSite site = new SPSite("http://devbook.sp2010.local/"))
            {
                // Get a reference to the BDC Service Application Proxy
                SPServiceContext serviceContext = SPServiceContext.GetContext(site);

                BdcServiceApplicationProxy bdcProxy =
                    (BdcServiceApplicationProxy)serviceContext.GetDefaultProxy(typeof(BdcServiceApplicationProxy));

                DatabaseBackedMetadataCatalog model = bdcProxy.GetDatabaseBackedMetadataCatalog();

                IEntity entity = model.GetEntity("http://schemas.devleap.com/WCFCustomerService", "WCFCustomer");
                ILobSystem ls = entity.GetLobSystem();

                ILobSystemInstance lsi =
                        ls.GetLobSystemInstances()["http://ws.devleap.local/WCFCustomerService/CustomersService.svc?WSDL"];
                IView creatorView = entity.GetCreatorView("AddCustomer");
                IFieldValueDictionary customerFields = creatorView.GetDefaultValues();

                customerFields["CustomerID"] = "DM001";
                customerFields["CompanyName"] = "DevLeap";
                customerFields["ContactName"] = "Paolo Pialorsi";
                customerFields["Country"] = "Italy";

                Identity identifierValues = entity.Create(customerFields, lsi);

                // Enumerate all the customers
                IMethodInstance method = entity.GetMethodInstance("ListAllCustomers", MethodInstanceType.Finder);
                IEntityInstanceEnumerator ieie = entity.FindFiltered(method.GetFilters(), lsi);
                IView view = entity.GetFinderView(method.Name);
                while (ieie.MoveNext())
                {
                    foreach (IField field in view.Fields)
                    {
                        if (ieie.Current[field] != null)
                        {
                            Console.WriteLine("{0}: {1}", field.Name, ieie.Current[field].ToString());
                        }
                    }
                }
            }
        }
    }
}
